uniform float4x4 sl_modelViewProj;
uniform float4 sl_textureBlend;

#ifdef DX9

TEXTURE gDiffuseMap;
TEXTURE gDiffuseMap2;

sampler gDiffuseSampler1 = sampler_state {
    Texture = (gDiffuseMap);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler gDiffuseSampler2 = sampler_state {
    Texture = (gDiffuseMap2);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};

#else

Texture2D gDiffuseMap;
Texture2D gDiffuseMap2;

SamplerState gTriLinearSamClamp {
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
};

#endif

void VS(  float4 position : POSITION,
          float4 color : COLOR,
          float4 texCoord : TEXCOORD,

          out float4 oPosition : SV_POSITION,
          out float4 oColor : COLOR,
          out float4 oTexCoord : TEXCOORD
       )
{
    oPosition = mul(sl_modelViewProj, position);
    oTexCoord = texCoord;
    oColor = color;
}

float4 PS(float4 posH : SV_POSITION,
          float4 color : COLOR,
          float4 texCoord : TEXCOORD ) : SV_TARGET {
    float4 fragIn;
#ifdef DX9
    fragIn = lerp(tex2D(gDiffuseSampler1, texCoord.xy).rgba, tex2D(gDiffuseSampler2, texCoord.xy).rgba, sl_textureBlend.x);
#else
    fragIn = lerp(gDiffuseMap.Sample(gTriLinearSamClamp, texCoord.xy), gDiffuseMap2.Sample(gTriLinearSamClamp, texCoord.xy), sl_textureBlend.x);
#endif
    return fragIn;
}

#ifdef DX11
technique11 ColorTech {
    pass P0
    {
        SetVertexShader( CompileShader( vs_5_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, PS() ) );
    }
}
#endif

#ifdef DX10
technique10 ColorTech {
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}
#endif

#ifdef DX10LEVEL9
technique10 ColorTech {
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0_level_9_1, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0_level_9_1, PS() ) );
    }
}
#endif

#ifdef DX9
technique {
    pass P0
    {
        SetVertexShader( CompileShader( vs_3_0, VS() ) );
        SetPixelShader( CompileShader( ps_3_0, PS() ) );
    }
}
#endif

#ifdef DX9_2_0
technique {
    pass P0
    {
        SetVertexShader( CompileShader( vs_2_0, VS() ) );
        SetPixelShader( CompileShader( ps_2_0, PS() ) );
    }
}
#endif
