uniform sampler2D sl_tex2D;
uniform mat4 sl_modelViewProj;
uniform mat4 sl_modelView;
uniform vec4 sl_displacementVectorAndContrast;
uniform vec4 sl_fadeAndDisplacementFactor;
uniform vec4 sl_upVectorAndThickness;
uniform vec4 sl_sunColor;
uniform vec4 sl_skyColor;
uniform vec4 sl_modelPos;
uniform vec4 sl_cameraPos;
uniform mat4 sl_invBasis;
uniform vec4 sl_layerSizeAndUnitScale;
uniform vec4 sl_fogColorAndDensity;
uniform vec4 sl_extinctionFactor;

varying vec3 V;
varying float fadeDist;

//#define FLATTEN_WITH_DISTANCE

vec4 overridePosition(in vec4 position);

void main()
{
    float unitScale = sl_layerSizeAndUnitScale.w;

    vec4 position = gl_Vertex;

    vec3 worldPos;
    vec4 w4;
#ifdef FLATTEN_WITH_DISTANCE
    worldPos = position.xyz + sl_modelPos.xyz;
    w4 = sl_invBasis * vec4(worldPos, 1.0);
    vec3 initV = (sl_cameraPos.xyz - w4.xyz);
    float vertDist = length(initV) * unitScale;

    float fogExponent = vertDist * sl_fogColorAndDensity.w;
    float fogFactor = clamp(exp(-abs(fogExponent * fogExponent)), 0.0, 1.0);
#endif

    gl_TexCoord[0] = gl_MultiTexCoord0;

    vec4 texel = texture2D(sl_tex2D, gl_MultiTexCoord0.xy);
    float minNSrc = sl_extinctionFactor.y;
    float thicknessFactor = texel.x * (1.0 - minNSrc) + minNSrc;

    float thickness = thicknessFactor * sl_upVectorAndThickness.w;

#ifdef FLATTEN_WITH_DISTANCE
    thickness = mix(sl_upVectorAndThickness.w, thickness, fogFactor);
#endif

    vec3 displacement = vec3(0.0,-1.0,0.0) * (sl_upVectorAndThickness.w - thickness);

    position.xyz += displacement;

    worldPos = position.xyz + sl_modelPos.xyz;

    w4 = sl_invBasis * vec4(worldPos, 1.0);
    V = (sl_cameraPos.xyz - w4.xyz);

    fadeDist = length( (vec4(V, 1.0) * sl_invBasis).xz );

    gl_FrontColor = gl_Color;

    gl_Position = overridePosition(sl_modelViewProj * position);
}
